import { ChainName, PlatformName } from "@wormhole-foundation/sdk-base";
import { Platform } from "./platform";
import { AutomaticTokenBridge, TokenBridge } from "./protocols/tokenBridge";
import { AutomaticCircleBridge, CircleBridge } from "./protocols/cctp";
import { IbcBridge } from "./protocols/ibc";
import { RpcConnection } from "./rpc";
import { SignedTx, TokenId } from "./types";
import { WormholeMessageId } from "./attestation";
export declare abstract class ChainContext<P extends PlatformName> {
    readonly chain: ChainName;
    protected rpc?: RpcConnection<P>;
    protected tokenBridge?: TokenBridge<P>;
    protected autoTokenBridge?: AutomaticTokenBridge<P>;
    protected circleBridge?: CircleBridge<P>;
    protected autoCircleBridge?: AutomaticCircleBridge<P>;
    protected ibcBridge?: IbcBridge<P>;
    abstract platform: Platform<P>;
    constructor(chain: ChainName);
    getRpc(): Promise<RpcConnection<P>>;
    getDecimals(token: TokenId | "native"): Promise<bigint>;
    getBalance(walletAddr: string, token: TokenId | "native"): Promise<bigint | null>;
    getCurrentBlock(): Promise<number>;
    parseTransaction(txid: string): Promise<WormholeMessageId[]>;
    sendWait(stxns: SignedTx): Promise<string[]>;
    supportsTokenBridge: () => boolean;
    getTokenBridge(): Promise<TokenBridge<P>>;
    supportsAutomaticTokenBridge: () => boolean;
    getAutomaticTokenBridge(): Promise<AutomaticTokenBridge<P>>;
    supportsCircleBridge: () => boolean;
    getCircleBridge(): Promise<CircleBridge<P>>;
    supportsAutomaticCircleBridge: () => boolean;
    getAutomaticCircleBridge(): Promise<AutomaticCircleBridge<P>>;
    supportsIbcBridge: () => boolean;
    getIbcBridge(): Promise<IbcBridge<P>>;
}
//# sourceMappingURL=chain.d.ts.map