import { ChainId, ChainName, PlatformName } from "@wormhole-foundation/sdk-base";
import { ChainAddress, NativeAddress, UniversalOrNative } from "../address";
import { IbcMessageId, WormholeMessageId } from "../attestation";
import { RpcConnection } from "../rpc";
import { TokenId, TxHash } from "../types";
import { UnsignedTransaction } from "../unsignedTransaction";
export type GatewayTransferDetails = {
    token: TokenId | "native";
    amount: bigint;
    from: ChainAddress;
    to: ChainAddress;
    nonce?: number;
    fee?: bigint;
    payload?: Uint8Array;
    nativeGas?: bigint;
};
export interface GatewayMsg {
    chain: ChainId;
    recipient: string;
    fee: string;
    nonce: number;
    payload?: string;
}
export interface GatewayTransferMsg {
    gateway_transfer: {
        chain: ChainId;
        recipient: string;
        fee: string;
        nonce: number;
    };
}
export interface GatewayTransferWithPayloadMsg {
    gateway_transfer_with_payload: {
        chain: ChainId;
        recipient: string;
        fee: string;
        nonce: number;
        payload: string;
    };
}
export interface GatewayIbcTransferMsg {
    gateway_ibc_token_bridge_payload: GatewayTransferMsg | GatewayTransferWithPayloadMsg;
}
export declare function isGatewayTransferMsg(thing: GatewayTransferMsg | any): thing is GatewayTransferMsg;
export declare function isGatewayTransferWithPayloadMsg(thing: GatewayTransferWithPayloadMsg | any): thing is GatewayTransferWithPayloadMsg;
export declare function isGatewayIbcTransferMsg(thing: GatewayIbcTransferMsg | any): thing is GatewayIbcTransferMsg;
export declare function isGatewayTransferDetails(thing: GatewayTransferDetails | any): thing is GatewayTransferDetails;
export declare function asGatewayMsg(msg: GatewayTransferMsg | GatewayTransferWithPayloadMsg | GatewayIbcTransferMsg | string): GatewayMsg;
export declare function gatewayTransferMsg(gtd: GatewayTransferDetails | GatewayMsg): GatewayTransferMsg | GatewayTransferWithPayloadMsg;
export declare function makeGatewayTransferMsg(chain: ChainName | ChainId, recipient: NativeAddress<"Cosmwasm"> | string, fee: bigint | undefined, nonce: number, payload?: string): GatewayTransferWithPayloadMsg | GatewayTransferMsg;
export interface IbcTransferInfo {
    id: IbcMessageId;
    data: IbcTransferData;
    pending: boolean;
}
export declare function isIbcTransferInfo(thing: IbcTransferInfo | any): thing is IbcTransferInfo;
export interface IbcTransferData {
    amount: string;
    denom: string;
    memo: string;
    receiver: string;
    sender: string;
}
export interface SupportsIbcBridge<P extends PlatformName> {
    getIbcBridge(rpc: RpcConnection<P>): Promise<IbcBridge<P>>;
}
export declare function supportsIbcBridge<P extends PlatformName>(thing: SupportsIbcBridge<P> | any): thing is SupportsIbcBridge<P>;
export interface IbcBridge<P extends PlatformName> {
    transfer(sender: UniversalOrNative<P>, recipient: ChainAddress, token: UniversalOrNative<P> | "native", amount: bigint, payload?: Uint8Array): AsyncGenerator<UnsignedTransaction>;
    lookupMessageFromIbcMsgId(msg: IbcMessageId): Promise<WormholeMessageId>;
    lookupTransferFromTx(txid: TxHash): Promise<IbcTransferInfo>;
    lookupTransferFromIbcMsgId(msg: IbcMessageId): Promise<IbcTransferInfo>;
    lookupTransferFromMsg(payload: GatewayTransferMsg | GatewayTransferWithPayloadMsg): Promise<IbcTransferInfo>;
}
//# sourceMappingURL=ibc.d.ts.map