import { chainItem, amountItem } from "../layout-items";
import { registerPayloadType } from "../vaa";
const bamAddressItem = {
    binary: "bytes",
    lengthSize: 2
};
const customOrEmpty = (custom) => custom ? { custom } : {};
const messageLayout = (type, customContents) => [
    { name: "magicByte", binary: "uint", size: 1, custom: 0xbb, omit: true },
    { name: "version", binary: "uint", size: 1, custom: 0, omit: true },
    { name: "type", binary: "uint", size: 1, custom: type, omit: true },
    { name: "index", binary: "uint", size: 8 },
    { name: "targetChain", ...chainItem() },
    { name: "targetAddress", ...bamAddressItem },
    { name: "senderAddress", ...bamAddressItem },
    { name: "contents", binary: "bytes", lengthSize: 2, ...customOrEmpty(customContents) },
];
const tokenMessageLayout = (custom) => [
    ...messageLayout(1, custom?.contents),
    { name: "bridge", binary: "uint", size: 1, ...customOrEmpty(custom?.bridge) },
    { name: "assetIdentifier", binary: "uint", size: 2, ...customOrEmpty(custom?.assetIdentifier) },
    { name: "amount", ...amountItem },
];
const extendedMessageLayout = (custom) => [
    ...messageLayout(2, custom?.contents),
    { name: "relaySignal", binary: "bytes", lengthSize: 2, ...customOrEmpty(custom?.relaySignal) },
];
export const bamPayloads = [
    ["BamMessage", messageLayout(0)],
    ["BamTokenMessage", tokenMessageLayout()],
    ["BamExtendedMessage", extendedMessageLayout()],
];
bamPayloads.forEach(([payloadLiteral, layout]) => registerPayloadType(payloadLiteral, layout));
//# sourceMappingURL=bam.js.map